from typing import Any, Callable, Generic, List, Dict, Text, Optional, Tuple, Union, TypeVar, Type
import base64

class ByteUtil:

    @staticmethod
    def to_base64(data: bytes) -> str:
        """
        Convert bytes to a Base64 encoded string.
        """
        return base64.b64encode(data).decode('utf-8')
    

    @staticmethod
    def from_base64(encoded_data: str) -> bytes:
        """
        Convert a Base64 encoded string back to bytes.
        """
        return base64.b64decode(encoded_data.encode('utf-8'), validate=True)


from typing import Any, Optional, Dict, Tuple, Text
import inspect


class DictionaryUtil:
    def __init__(self):
        pass
    

    @staticmethod
    def get(dict:Dict, name:str, default:Any) -> Any:
        return dict[name] if (name in dict) else default


    @staticmethod
    def to_dict(obj) -> Dict[str, Any]:
        d = {}
        for name in dir(obj):
            value = getattr(obj, name)
            if not name.startswith('__') and not inspect.ismethod(value):
                d[name] = value
        return d
    
    
    @staticmethod
    def convert(model:Any) -> Dict:
        d = dict((name, getattr(model, name)) for name in dir(model) if not name.startswith('__') and not inspect.ismethod(name))

        return d


    """ NOT WORKING
    # Support recursive class structure as well as datetime
    @staticmethod
    def convert(model:Any) -> Dict:
        return json.loads(
            json.dumps(model, indent=4, separators=(',', ': ')), cls=DateTimeDecoder)
    """


from typing import Any, Optional, Dict, Tuple, Text, List


class ClassUtil:
    @staticmethod
    def merge(class_a:Any, class_b:Any) -> Any:
        class_a.__dict__.update(class_b.__dict__)
        return class_a
    

    @staticmethod
    def get_class_name(cls:Any) -> str:
       return cls.__name__ if cls is not None else ""


    @staticmethod
    def get_class_instance_name(cls:Any) -> str:
       return cls.__class__.__name__ if cls is not None else ""


    @staticmethod
    def get_class_names(model: Any) -> List[str]:
        class_names = set()

        # Add class name of the model itself
        class_names.add(type(model).__name__)

        # Get attributes of the model
        try:
            attributes = vars(model)
        except TypeError:
            # model has no __dict__ (e.g. primitive)
            return List(class_names)

        for attr_name, attr_value in attributes.items():
            if attr_value is None:
                continue

            # Check if it's a list and has elements
            if isinstance(attr_value, list) and len(attr_value) > 0:
                first_elem = attr_value[0]
                elem_class_name = type(first_elem).__name__
                if elem_class_name not in ["dict", "list", "str", "int", "float", "bool"]:
                    class_names.add(elem_class_name)

            # Check if it's another object (not primitive, not list)
            elif hasattr(attr_value, "__dict__"):
                value_class_name = type(attr_value).__name__
                if value_class_name not in ["dict", "list", "str", "int", "float", "bool"]:
                    class_names.add(value_class_name)

        return list(class_names)